'use strict';

class Api extends Base {
  constructor() {
    super();
    settings.register(Api.SETTING_CLIENT_API_KEY, Api.clientApiKey);
  }

  registerListeners() {
    window.addEventListener('offline', this.onNetworkStateChanged_.bind(this),
                            false);
    window.addEventListener('online', this.onNetworkStateChanged_.bind(this),
                            false);
  }

  initialize() {
    super.initialize();
    this.onNetworkStateChanged_();
  }

  static get deviceHash() {
    return App.browserType === App.BROWSER_TYPE_OPERA ? Api.DEVICE_HASH_OPERA :
        Api.DEVICE_HASH_CHROME;
  }

  static get clientType() {
    return App.browserType === App.BROWSER_TYPE_OPERA ? Api.CLIENT_TYPE_OPERA :
        Api.CLIENT_TYPE_CHROME;
  }

  static get clientApiKey() {
    return App.browserType === App.BROWSER_TYPE_OPERA ?
        Api.CLIENT_API_KEY_OPERA : Api.CLIENT_API_KEY_CHROME;
  }

  static get key() {
    return settings.get (Api.SETTING_CLIENT_API_KEY) || Api.clientApiKey;
  }

  static get isUsingStaging() {
    return Api.key !== Api.clientApiKey;
  }

  get isOnLine() {
    return navigator.onLine;
  }

  delayRequest_(url, options, version, delay, onlyOnce) {
    return new Promise(function(resolve, reject) {
      onlyOnce = onlyOnce === undefined ? false : onlyOnce;

      setTimeout(function() {
        this.request(url, options, version, !onlyOnce).then(resolve)
            .catch(reject);
      }.bind(this), delay);
    }.bind(this));
  }

  sendRequest_(url, options, version) {
    return new Promise(function(resolve, reject) {
      var data = new FormData();
      for (var param in options) {
        data.append(param, options[param]);
      }

      var xhr = new XMLHttpRequest();
      xhr.open('POST', [Url.API, version, url].join('/'));
      xhr.withCredentials = true;
      xhr.timeout = Api.TIMEOUT_SERVER;
      xhr.responseType = 'json';
      xhr.setRequestHeader('SE-Client-Type', Api.clientType);
      xhr.setRequestHeader('SE-Client-Name', App.id);
      xhr.setRequestHeader('SE-Client-API-Key', Api.key);
      xhr.setRequestHeader('SE-Client-Version', App.version);
      xhr.setRequestHeader('SE-Client-Locale', i18n.lang);
      xhr.setRequestHeader('SE-Operating-System', navigator.userAgent);

      xhr.ontimeout = function () {
        reject({
          'type': Api.ERROR_SOURCE_TIMEOUT,
          'status': xhr.status
        });
      };

      xhr.onerror = function () {
        reject({
          'type': Api.ERROR_SOURCE_NETWORK,
          'status': xhr.status
        });
      };

      xhr.onload = function () {
        if (xhr.status !== 200 || !xhr.response || !xhr.response.return_code) {
          reject({
            'type': xhr.status === 200 ? Api.ERROR_SOURCE_NETWORK :
                Api.ERROR_SOURCE_SERVER,
            'status': xhr.status
          });
        } else {
          resolve(xhr.response);
        }
      };

      xhr.send(data);
    }.bind(this));
  }

  request(url, options, version, canDelay) {
    if (!proxy.isControlled || errors.isActiveErrorCritical) {
      return Promise.reject();
    }

    canDelay = canDelay === undefined ? true : canDelay;
    version = version || Api.DEFAULT_VERSION;
    return new Promise(function(resolve, reject) {
      sessionCookie.check().then(function () {
        this.sendRequest_(url, options, version).then(function(response) {
          var requestStatus = response.return_code;
          if (requestStatus.hasOwnProperty(Api.CODE_OK)) {
            resolve(response.data);
          } else if (requestStatus.hasOwnProperty(Api.CODE_TOO_MANY_REQUESTS)) {
            this.delayRequest_(url, options, version, Api.TIMEOUT)
                .then(resolve).catch(reject);
          } else if (requestStatus.hasOwnProperty(Api.CODE_WRONG_DEVICE)) {
            Api.Events.onDeviceError.dispatch(this);
            reject();
          } else if (requestStatus.hasOwnProperty(Api.CODE_UNKNOWN_USER)) {
            Api.Events.onUserError.dispatch(this);
            reject();
          } else {
            for (let code in requestStatus) {
              analytics.countEvent(Analytics.EVENT_LOCAL_ERROR_CODE, code,
                  requestStatus[code]);
              let message = i18n.get('errorCode' + code, [Url.SUPPORT_EMAIL]);
              if (!message) {
                message = '[' + code + '] ' + i18n.get('errorCodeDefault',
                                                       [Url.SUPPORT_EMAIL]);
                analytics.countEvent(Analytics.EVENT_NOT_TRANSLATED, code,
                    requestStatus[code]);
              }

              requestStatus[code] = message;
            }

            reject(requestStatus);
          }
        }.bind(this)).catch(function(details) {
          if (canDelay) {
            this.delayRequest_(url, options, version, Api.TIMEOUT / 3, true)
                .then(resolve).catch(reject);
            return;
          }

          var type = details.type;
          var status = details.status;
          var message = {};
          if (type === Api.ERROR_SOURCE_NETWORK) {
            message[Errors.CODE_API_NETWORK] =
                i18n.get('errorMessageNetwork');
          } else {
            message[Errors.CODE_API_SERVER] = i18n.get('errorMessageServer');
            var label = url;
            if (type === Api.ERROR_SOURCE_SERVER && status !== 200) {
              label = '[' + status + '] ' + label;
            }

            analytics.countEvent(Analytics.EVENT_COMMUNICATION_ERROR, type,
                                 label);
          }
          reject(message);
        }.bind(this));
      }.bind(this)).catch(reject);
    }.bind(this));
  }

  register(email, password) {
    return new Promise(function(resolve, reject) {
      if (!email || !password) {
        reject();
      } else {
        this.request('register_subscriber', {
          'email': email,
          'password': password,
          'locale': i18n.lang
        }).then(resolve).catch(reject);
      }
    }.bind(this));
  }

  login(email, password) {
    return new Promise(function(resolve, reject) {
      if (!email || !password) {
        reject();
      } else {
        this.request('subscriber_login', {
          'login': email,
          'password': password,
          'client_type': Api.clientType
        }).then(resolve).catch(reject);
      }
    }.bind(this));
  }

  logout() {
    return sessionCookie.remove();
  }

  usage(deviceId) {
    return new Promise(function(resolve, reject) {
      if (!deviceId) {
        reject();
      } else {
        this.request('subscriber_usage', {
          'device_id': hash(deviceId)
        }).then(resolve).catch(reject);
      }
    }.bind(this));
  }

  deviceRegister(name) {
    return new Promise(function(resolve, reject) {
      if (!name) {
        reject();
      } else {
        this.request('register_device', {
          'device_name': name,
          'device_hash': Api.deviceHash,
          'client_type': Api.clientType
        }, Api.DEFAULT_VERSION, false).then(function (data) {
          this.getProxyServerPattern_(data);
          if (Boolean(data) && data.hasOwnProperty('device_id') &&
              data.hasOwnProperty('device_password')) {
            resolve({
              'id': data.device_id,
              'password': data.device_password
            });
          } else {
            reject();
          }
        }.bind(this)).catch(reject);
      }
    }.bind(this));
  }

  deviceVerify(id, password) {
    return new Promise(function(resolve, reject) {
      if (!id) {
        reject();
      } else {
        this.request('device_login', {
          'device_hash': Api.deviceHash,
          'device_id': hash(id),
          'password': password
        }).then(function(data) {
          this.getProxyServerPattern_(data);
          resolve();
        }.bind(this)).catch(reject);
      }
    }.bind(this));
  }

  deviceGeneratePassword(id) {
    return new Promise(function(resolve, reject) {
      if (!id) {
        reject();
      } else {
        this.request('device_generate_password', {
          'device_id': hash(id)
        }).then(function(data) {
          this.getProxyServerPattern_(data);
          if (Boolean(data) && data.hasOwnProperty('device_password')) {
            resolve(data.device_password);
          } else {
            reject();
          }
        }.bind(this)).catch(reject);
      }
    }.bind(this));
  }

  deviceLogout(id) {
    return new Promise(function(resolve, reject) {
      if (!id) {
        reject();
      } else {
        var onPasswordRemoved = new Promise(function(resolve, reject) {
          this.deviceGeneratePassword(id).then(resolve).catch(resolve);
        }.bind(this));

        onPasswordRemoved.then(bindOnce(this.logout, this)).then(resolve)
            .catch(reject);
      }
    }.bind(this));
  }

  rewards() {
    return new Promise(function(resolve, reject) {
      this.request('subscriber_rewards', {}, 'v3').then(function(data) {
        if (Boolean(data) && data.hasOwnProperty('rewards')) {
          resolve(data.rewards);
        } else {
          reject();
        }
      }).catch(reject);
    }.bind(this));
  }

  rewardMarkAsCompleted(code) {
    return new Promise(function(resolve, reject) {
      if (!code) {
        reject();
      } else {
        this.request('subscriber_reward_completed', {
          code: code
        }).then(resolve).catch(reject);
      }
    }.bind(this));
  }

  geoServerLocation() {
    return new Promise(function(resolve, reject) {
      this.request('geo_lookup', {}, 'v3').then(function(data) {
        if (Boolean(data) && data.hasOwnProperty('geo') &&
            data.in_surfeasy_network === true) {
          resolve(data.geo);
        } else {
          reject();
        }
      }).catch(reject);
    }.bind(this));
  }

  geoUserLocation() {
    return new Promise(function(resolve, reject) {
      this.request('geo_lookup?direct', {}, 'v3', false).then(function(data) {
        if (Boolean(data) && data.hasOwnProperty('geo')) {
          if (data.in_surfeasy_network === true) {
            analytics.countEvent(Analytics.EVENT_DOUBLE_PROXY, true);
          }
          resolve(data.geo);
        } else {
          reject();
        }
      }).catch(reject);
    }.bind(this));
  }

  geoList(deviceId) {
    return new Promise(function(resolve, reject) {
      if (!deviceId) {
        reject();
      } else {
        this.request('geo_list', {
          'device_id': hash(deviceId)
        }).then(function(data) {
          if (Boolean(data) && data.hasOwnProperty('geos')) {
            resolve(data.geos);
          } else {
            reject();
          }
        }).catch(reject);
      }
    }.bind(this));
  }

  serviceCheck() {
    return new Promise(function(resolve, reject) {
      this.request('service_check', {}, Api.DEFAULT_VERSION, false).then(
          function() {
            resolve(true);
          }).catch(function(details) {
        if (details && details.hasOwnProperty(Api.CODE_OUTSIDE_PROXY)) {
          resolve(false);
        } else {
          reject(details);
        }
      });
    }.bind(this));
  }

  sendSMS(phone) {
    return new Promise(function(resolve, reject) {
      if (!phone) {
        reject();
      } else {
        this.request('send_sms', {
          'phone_number': phone
        }).then(resolve).catch(reject);
      }
    }.bind(this));
  }

  getAutoLoginToken() {
    return new Promise(function(resolve, reject) {
      this.request('get_auto_login_token', {}).then(resolve).catch(reject);
    }.bind(this));
  }

  getProxyServerPattern_(data) {
    if (data && data.hasOwnProperty('dns_format') &&
        data.dns_format.hasOwnProperty('sub')) {
      Api.Events.onProxyServerPatternReceived.dispatch(data.dns_format.sub);
    }
  }

  onNetworkStateChanged_() {
    Api.Events.onNetworkStateChanged.dispatch(this);
  }
};

Api.Events = {
  onUserError: new chrome.Event(),
  onDeviceError: new chrome.Event(),
  onProxyServerPatternReceived: new chrome.Event(),
  onNetworkStateChanged: new chrome.Event()
};

Api.SETTING_CLIENT_API_KEY = 'api.clientApiKey';

Api.DEFAULT_VERSION = 'v2';
Api.CLIENT_TYPE_OPERA = 'se0204';
Api.CLIENT_TYPE_CHROME = 'se0205';

Api.CLIENT_API_KEY_OPERA =
    '184E166877D1AA1AB7BB702E31DE0EE650442D67529CFEE7DA12FE72455EF32D';
Api.CLIENT_API_KEY_CHROME =
    '37403FC0A1C5C5FCDAAF7A31253A855AC900BEE2CFF2AB3FFF5AEFD83ECB04F9';
Api.DEVICE_HASH_OPERA = hash('OPERA');
Api.DEVICE_HASH_CHROME = hash('CHROME');

Api.ERROR_SOURCE_SERVER = 'server';
Api.ERROR_SOURCE_NETWORK = 'network';
Api.ERROR_SOURCE_TIMEOUT = 'timeout';

Api.CODE_OK = 0;
Api.CODE_TOO_MANY_REQUESTS = 12;
Api.CODE_WRONG_DEVICE = 9;
Api.CODE_UNKNOWN_USER = 21;
Api.CODE_OUTSIDE_PROXY = 28;
Api.TIMEOUT = 5 * 1000;
Api.TIMEOUT_SERVER = 30 * 1000 + Api.TIMEOUT;

window.Api = Api;
