'use strict';

class App {
  constructor() {
    settings.register(App.SETTING_IS_ONBOARDING_COMPLETED, false);
    settings.register(App.SETTING_IS_LANDING_PAGE_SHOWN, false);
    settings.register(App.SETTING_LAST_RELOAD, 0);

    this.isReady_ = false;
  }

  registerListeners() {
    Settings.Events.onChanged.addListener(this.onSettingChanged_.bind(this));
  }

  static get browserType() {
    return /OPR/gi.test(navigator.userAgent) ? App.BROWSER_TYPE_OPERA :
        App.BROWSER_TYPE_CHROME;
  }

  static get id() {
    return chrome.runtime.id;
  }

  static get version() {
    return chrome.app.getDetails().version;
  }

  get isPopupVisible() {
    var popupsViews = chrome.extension.getViews({
      'type': chrome.extension.ViewType.POPUP
    });

    return Boolean(popupsViews.length);
  }

  get isReady() {
    return this.isReady_;
  }

  set isReady(value) {
    if (!this.isReady_ && value) {
      this.isReady_ = true;
      analytics.countPage([App.ID, App.version]);
      this.initialize();
      App.Events.onReady.dispatch(this);
    }
  }

  get isOnboardingCompleted() {
    return settings.get(App.SETTING_IS_ONBOARDING_COMPLETED);
  }

  set isOnboardingCompleted(value) {
    settings.set(App.SETTING_IS_ONBOARDING_COMPLETED, value);
  }

  get isLandingPageShown() {
    return settings.get(App.SETTING_IS_LANDING_PAGE_SHOWN);
  }

  set isLandingPageShown(value) {
    settings.set(App.SETTING_IS_LANDING_PAGE_SHOWN, value);
  }

  report() {
    var data = {
      'settings': localStorage,
      'networkTasks': networkTasks,
      'app': app,
      'api': api,
      'offroad': offroad,
      'device': device,
      'dataPlan': dataPlan,
      'geo': geo,
      'rewards': rewards,
      'user': user,
      'proxy': proxy,
      'errors': errors,
      'browsingActivity': browsingActivity,
      'notifications': notifications,
      'toolbarState': toolbarState
    };

    data = JSON.stringify(data);
    Url.open('data:application/json;base64,' +
        btoa(unescape(encodeURIComponent(data))));
  }

  restart() {
    var isLandingPageVisible = chrome.extension.getViews().some(function(view) {
      return view.location.href.indexOf(Url.LANDING_PAGE) !== -1;
    });

    if (isLandingPageVisible) {
      this.isLandingPageShown = false;
    }

    chrome.runtime.reload();
  }

  reset() {
    user.logout();
    localStorage.clear();
    this.restart();
  }

  disableProxyExtensions() {
    chrome.management.getAll(function(items) {
      items = items.filter(function(item) {
        return item.enabled && item.id !== chrome.runtime.id &&
            item.permissions.indexOf('proxy') !== -1;
      });
      items.forEach(function(item) {
        chrome.management.setEnabled(item.id, false);
      });
    });
  }

  onSettingChanged_(setting) {
    if (!setting) {
      return;
    }

    if (setting.key === App.SETTING_IS_ONBOARDING_COMPLETED) {
      App.Events.onOnboardingCompleteChanged.dispatch(this);
    }
  }

  initialize() {
    this.registerListeners();

    chrome.runtime.setUninstallURL(Url.UNINSTALL);

    if (!this.isLandingPageShown) {
      if (!user.isLoggedIn || !rewards.areWelcomeRewardsCompleted) {
        Url.openOrSelect(chrome.extension.getURL(Url.LANDING_PAGE));
      }

      this.isLandingPageShown = true;
    }
  }
};


App.ID = 'app';
App.SETTING_IS_ONBOARDING_COMPLETED = 'app.isOnboardingCompleted';
App.SETTING_IS_LANDING_PAGE_SHOWN = 'app.isLandingPageShown';
App.SETTING_LAST_RELOAD = 'app.lastReload';

App.BROWSER_TYPE_OPERA = 'opera';
App.BROWSER_TYPE_CHROME = 'chrome';

App.Events = {
  onReady: new chrome.Event(),
  onCookiesError: new chrome.Event(),
  onOnboardingCompleteChanged: new chrome.Event()
};

window.App = App;
