'use strict';

class Device extends Base {
  constructor() {
    super();
    settings.register(Device.SETTING_ID, null);
    settings.register(Device.SETTING_PASSWORD, null);

    this.isLimitReached_ = false;
    this.task_ = new NetworkTask(Device.TASK_ID, this.onRequest_.bind(this),
                                 this.onCanRunTask_.bind(this));
    this.task_.isCacheable = false;
    networkTasks.add(this.task_);
  }

  registerListeners() {
    super.registerListeners();
    BrowsingActivity.Events.onDeviceRemovedAction.addListener(
        this.onDeviceRemoved_.bind(this));
    User.Events.onLoggedIn.addListener(this.onUserLoggedIn_.bind(this));
    Api.Events.onDeviceError.addListener(this.onDeviceError_.bind(this));
    User.Events.onLoggedOut.addListener(this.onUserLoggedOut_.bind(this));
    Proxy.Events.onError.addListener(this.onProxyError_.bind(this));
  }

  initialize() {
    super.initialize();
  }

  get isLimitReached() {
    return this.isLimitReached_;
  }

  set isLimitReached(value) {
    if (this.isLimitReached === value) {
      return;
    }

    this.isLimitReached_ = value;

    Device.Events.onChanged.dispatch(this);
  }

  get isValid() {
    return Boolean(this.id) && Boolean(this.password);
  }

  get isConnected() {
    return this.isValid && user.isLoggedIn;
  }

  get id() {
    return settings.get(Device.SETTING_ID);
  }

  set id(value) {
    if (this.id === value) {
      return;
    }

    settings.set(Device.SETTING_ID, value);

    Device.Events.onChanged.dispatch(this);
  }

  get password() {
    return settings.get(Device.SETTING_PASSWORD);
  }

  set password(value) {
    if (this.password === value) {
      return;
    }

    settings.set(Device.SETTING_PASSWORD, value);

    Device.Events.onChanged.dispatch(this);
  }

  checkLimit() {
    if (this.isLimitReached) {
      this.isLimitReached = false;
      this.task_.run();
    }
  }

  onCanRunTask_() {
    return user.isLoggedIn && !this.isLimitReached;
  }

  onRequest_(resolve, reject) {
    (new Promise(function(resolve, reject) {
      if (this.id) {
        if (this.password) {
          this.verifyTask_().then(resolve).catch(reject);
        } else {
          this.regeneratePasswordTask_().then(resolve).catch(reject);
        }
      } else {
        this.registerTask_().then(resolve).catch(reject);
      }
    }.bind(this))).then(function() {
      resolve(new NetworkTaskData(null, 5 * 60 * 1000));
    }).catch(reject);
  }

  registerTask_() {
    return new Promise(function(resolve, reject) {
      this.isLimitReached = false;
      this.generateName().then(function(name) {
        api.deviceRegister(name).then(function (device) {
          if (!this.task_.canRun) {
            reject();
            return;
          }
          this.id = device.id;
          this.password = device.password;
          resolve();
        }.bind(this)).catch(function (details) {
          if (details &&
              details.hasOwnProperty(Device.CODE_MAX_LIMIT_REACHED)) {
            this.isLimitReached = true;
            reject();
          } else {
            reject(details);
          }
        }.bind(this));
      }.bind(this));
    }.bind(this));
  }

  regeneratePasswordTask_() {
    return new Promise(function(resolve, reject) {
      if (!this.id) {
        return reject();
      } else {
        api.deviceGeneratePassword(this.id).then(function(password) {
          if (!this.task_.canRun) {
            reject();
            return;
          }
          this.password = password;
          resolve();
        }.bind(this)).catch(reject);
      }
    }.bind(this));
  }

  verifyTask_() {
    return new Promise(function(resolve, reject) {
      if (!this.isValid) {
        reject();
      } else {
        api.deviceVerify(this.id, this.password).then(resolve).catch(
            function(details) {
              if (details &&
                  (details.hasOwnProperty(Device.CODE_INCORRECT_PASSWORD) ||
                  details.hasOwnProperty(Device.CODE_INVALID_PASSWORD))) {
                this.password = null;
                reject();
              } else {
                reject(details);
              }
            }.bind(this));
      }
    }.bind(this));
  }

  onProxyError_() {
    this.task_.run();
  }

  onDeviceError_() {
    var isValid = this.isValid;

    this.isLimitReached = false;
    this.id = null;
    this.password = null;

    if (isValid || user.isAutoLogin) {
      user.logout();
    } else {
      this.task_.run();
    }
  }

  onDeviceRemoved_() {
    if (this.isValid) {
      this.task_.run();
    } else {
      this.checkLimit();
    }
  }

  onUserLoggedIn_() {
    this.task_.run();
    if (this.isValid) {
      Device.Events.onChanged.dispatch(this);
    }
  }

  onUserLoggedOut_() {
    this.isLimitReached = false;
    this.password = null;
    this.task_.clear();

    api.deviceLogout(this.id).catch(doNothing);
  }

  generateName() {
    return new Promise(function(resolve, reject) {
      var name = Device.NAME_TEMPLETE;
      chrome.runtime.getPlatformInfo(function(info) {
        var browser = App.browserType === App.BROWSER_TYPE_OPERA ? 'Opera' :
            'Chrome';

        var system = info.os === chrome.runtime.PlatformOs.WIN ? 'Windows' :
            (info.os === chrome.runtime.PlatformOs.MAC ? 'Mac OS X' :
                (info.os.charAt(0).toUpperCase() + info.os.substring(1)));

        name = name.replace('$browser$', browser);
        name = name.replace('$system$', system);

        name = name.replace('$time$', (new Date()).toLocaleString(i18n.lang, {
          'day': '2-digit',
          'month': 'short',
          'year': 'numeric'
        }));

        resolve(name);
      });
    }.bind(this));
  }
};

Device.Events = {
  onChanged: new chrome.Event()
};

Device.TASK_ID = 'device';

Device.SETTING_ID = 'device.id';
Device.SETTING_PASSWORD = 'device.password';
Device.CODE_WRONG_DEVICE = Api.CODE_WRONG_DEVICE;
Device.CODE_INVALID_PASSWORD = 7;
Device.CODE_INCORRECT_PASSWORD = 10;
Device.CODE_MAX_LIMIT_REACHED = 610;

Device.NAME_TEMPLETE = '$browser$ ($system$, $time$)';

window.Device = Device;
