'use strict';

class Geo extends Base {
  constructor() {
    super();
    settings.register(Geo.SETTING_COUNTRY_CODE, Geo.DEFAULT_COUNTRY_CODE);

    this.ipUserTask_ = new NetworkTask(Geo.TASK_ID_IP_USER,
                                       this.onIpUserRequest_.bind(this),
                                       this.canRunTask_.bind(this));
    networkTasks.add(this.ipUserTask_);
    this.userLocationUnknown_ = false;

    this.ipServerTask_ = new NetworkTask(Geo.TASK_ID_IP_SERVER,
                                         this.onIpServerRequest_.bind(this),
                                         this.canRunIpServerTask_.bind(this));
    this.ipServerTask_.isCacheable = false;
    networkTasks.add(this.ipServerTask_);

    this.listTask_ = new NetworkTask(Geo.TASK_ID_LIST,
                                     this.onListRequest_.bind(this),
                                     this.canRunTask_.bind(this));
    networkTasks.add(this.listTask_);

  }

  registerListeners() {
    super.registerListeners();

    Device.Events.onChanged.addListener(this.onDeviceChanged_.bind(this));
    NetworkTask.Events.onData.addListener(this.onNetworkDataChanged_.bind(this));
    Proxy.Events.onStateChanged.addListener(this.onProxyChanged_.bind(this));
    Proxy.Events.onError.addListener(this.onProxyError_.bind(this));
    Settings.Events.onChanged.addListener(this.onSettingChanged_.bind(this));
  }

  initialize() {
    super.initialize();
    this.onProxyChanged_();
  }

  get countryName() {
    var code = this.countryCode;
    var geo = this.list.reduce(function(current, next) {
      return !current && next.country_code === code ? next : current;
    }, null);

    if (geo) {
      return geo.country;
    }
  }

  get countryCode() {
    var code = null;
    var geo  = null;

    code = settings.get(Geo.SETTING_COUNTRY_CODE);
    if (code) {
      geo = this.list.reduce(function(current, next) {
        return !current && next.country_code === code ? next : current;
      }, null);

      if (geo) {
        return code;
      }
    }

    return Geo.DEFAULT_COUNTRY_CODE;
  }

  set countryCode(value) {
    if (this.countryCode === value) {
      return;
    }

    this.ipServerTask_.clear();
    settings.set(Geo.SETTING_COUNTRY_CODE, value);
  }

  get defaultServer() {
    return {
      'country_code': Geo.DEFAULT_COUNTRY_CODE,
      'country': i18n.get('locationProxyServerOptimized')
    }
  }

  get list() {
    return [this.defaultServer].concat(
        Array.isArray(this.listTask_.data) ? this.listTask_.data : []);
  }

  get serverLocation() {
    return this.ipServerTask_.data;
  }

  get userLocation() {
    return this.ipUserTask_.data;
  }

  get userLocationUnknown() {
    return this.userLocation === null && this.userLocationUnknown_;
  }

  set userLocationUnknown(value) {
    if (this.userLocationUnknown_ === value) {
      return;
    }

    this.userLocationUnknown_ = value;
    Geo.Events.onChanged.dispatch(this);
  }

  canRunTask_() {
    return device.isConnected;
  }

  canRunIpServerTask_() {
    return proxy.isConnected && this.canRunTask_();
  }

  onListRequest_(resolve, reject) {
    api.geoList(device.id).then(function(geos) {
      resolve(new NetworkTaskData(geos, 5 * 60 * 1000));
    }).catch(reject);
  }

  onIpUserRequest_(resolve, reject) {
    api.geoUserLocation().then(function(geo) {
      this.userLocationUnknown = false;
      resolve(new NetworkTaskData(geo, 3 * 60 * 1000));
    }.bind(this)).catch(function() {
      this.userLocationUnknown = true;
      // user ip can be null because direct API calls are blocked.
      resolve(new NetworkTaskData(null, 1 * 60 * 1000));
    }.bind(this));
  }

  onIpServerRequest_(resolve, reject) {
    api.geoServerLocation().then(function(geo) {
      resolve(new NetworkTaskData(geo, 1 * 60 * 1000));
    }).catch(reject);
  }

  onDeviceChanged_() {
    if (device.isConnected) {
      this.ipUserTask_.run();
      this.listTask_.run();
    } else {
      this.ipUserTask_.clear();
      this.listTask_.clear();
      this.ipServerTask_.clear();
    }
  }

  onProxyError_() {
    this.listTask_.run();
  }

  onProxyChanged_() {
    if (proxy.isConnected) {
      this.ipServerTask_.run();
    } else {
      this.ipServerTask_.clear();
    }
  }

  onSettingChanged_(setting) {
    if (setting.key === Geo.SETTING_COUNTRY_CODE) {
      Geo.Events.onChanged.dispatch(this);
    }
  }

  onNetworkDataChanged_(item) {
    if (item.name === Geo.TASK_ID_IP_USER ||
        item.name === Geo.TASK_ID_IP_SERVER ||
        item.name === Geo.TASK_ID_LIST) {
      Geo.Events.onChanged.dispatch(this);
    }
  }
};

Geo.Events = {
  onChanged: new chrome.Event()
};

Geo.TASK_ID_IP_USER = 'geoIpUser';
Geo.TASK_ID_IP_SERVER = 'geoIpServer';
Geo.TASK_ID_LIST = 'geoList';

Geo.SETTING_COUNTRY_CODE = 'geo.countryCode';

Geo.DEFAULT_COUNTRY_CODE = 'optimized';

window.Geo = Geo;
