'use strict';

class NetworkTasks extends Base {
  constructor() {
    super();
    this.tasks_ = [];
    this.isRunning_ = false;
    this.isRunPending_ = false;
    this.runTimeout_ = null;
    this.lastError_ = null;
    this.serverErrorsCount_ = 0;
    this.networkErrorsCount_ = 0;
  }

  registerListeners() {
    Errors.Events.onCriticalErrorFixed.addListener(this.run.bind(this));
    NetworkTask.Events.onFail.addListener(this.onRunFailed_.bind(this));
    User.Events.onLoggedOut.addListener(this.clearActiveIssues_.bind(this));
  }

  initialize() {
    super.initialize();
    this.run();
  }

  get lastError() {
    return this.lastError_;
  }

  get isRunning() {
    return this.isRunning_;
  }

  add(task) {
    this.tasks_.push(task);
  }

  remove(task) {
    var index = this.tasks_.indexOf(task);
    if (index !== -1) {
      this.tasks_.splice(index, 1);
    }
  }

  task(name) {
    return this.tasks_.filter(function(task) {
      return task.name === name;
    })[0];
  }

  run() {
    if (errors.isActiveErrorCritical) {
      this.runLater();
      return;
    }

    if (this.isRunning) {
      this.isRunPending_ = true;
      return;
    }

    if (this.runTimeout_) {
      window.clearTimeout(this.runTimeout_);
      this.runTimeout_ = null;
    }

    this.isRunning_ = true;

    Promise.all(this.tasks_.map(function(task) {
      return task.isValid ? Promise.resolve() : task.run();
    })).then(function() {
      this.clearActiveIssues_();
      this.onRunCompleted_();
    }.bind(this)).catch(this.onRunCompleted_.bind(this));
  }

  runAgain() {
    this.clearActiveIssues_();
    this.run();
  }

  runLater() {
    if (this.isRunning_) {
      return;
    }

    if (this.runTimeout_) {
      window.clearTimeout(this.runTimeout_);
      this.runTimeout_ = null;
    }

    this.runTimeout_ = setTimeout(this.run.bind(this), Api.TIMEOUT);
  }

  onRunCompleted_() {
    if (!this.isRunning_) {
      return;
    }
    this.isRunning_ = false;
    if (this.isRunPending_) {
      this.isRunPending_ = false;
      this.run();
    } else {
      this.runLater();
    }
  }

  onRunFailed_(ev) {
    if (this.tasks_.indexOf(ev.target) === -1 || !ev.details) {
      return;
    }

    var code = Object.keys(ev.details)[0];
    if (code > 0) {
      this.lastError_ = ev.details[code];
      NetworkTasks.Events.onError.dispatch(this);
    } else if (code === Errors.CODE_API_SERVER) {
      this.serverErrorsCount_++;
      if (this.serverErrorsCount_ > 20) {
        this.lastError_ = ev.details[code];
        NetworkTasks.Events.onError.dispatch(this);
        this.serverErrorsCount_ = 0;
      }
    } else if (code === Errors.CODE_API_NETWORK) {
      this.networkErrorsCount_++;
      if (this.networkErrorsCount_ > 20) {
        this.lastError_ = ev.details[code];
        NetworkTasks.Events.onError.dispatch(this);
        this.networkErrorsCount_ = 0;
      }
    }
  }

  clearActiveIssues_() {
    this.serverErrorsCount_ = 0;
    this.networkErrorsCount_ = 0;

    if (this.lastError_ !== null) {
      this.lastError_ = null;
      errors.reload();
    }
  }
}

NetworkTasks.Events = {
  onError: new chrome.Event()
};
