'use strict';

class Notifications extends Base {
  constructor() {
    super();

    settings.register(Notifications.CONNECTED_SHOWN, false);
    this.lastUsedDataPercent_ = null;
    this.isDataPlanOverBandwidth_ = false;
    this.isDataPlanExpired_ = false;
    this.items_ = [];

    this.hideAll();

    this.items_.push(new Notification({
      'id': Notification.ID_CONNECTED,
      'group': Notifications.GROUP_CONNECTION,
      'title': i18n.get('notificationsConnectedTitle'),
      'message': i18n.get('notificationsConnectedMessage')
    }));

    this.items_.push(new Notification({
      'id': Notification.ID_EXPIRED,
      'group': Notifications.GROUP_USAGE,
      'title': i18n.get('notificationsExpiredTitle'),
      'message': i18n.get('notificationsExpiredMessage')
    }));

    this.items_.push(new Notification({
      'id': Notification.ID_OVER_BANDWIDTH,
      'group': Notifications.GROUP_USAGE,
      'title': i18n.get('notificationsOverBandwidthTitle'),
      'message': i18n.get('notificationsOverBandwidthMessage')
    }));

    this.items_.push(new Notification({
     'id': Notification.ID_USED,
     'group': Notifications.GROUP_USAGE,
     'title': '',
     'message': i18n.get('notificationsUsageMessage')
     }));
  }

  registerListeners() {
    super.registerListeners();

    User.Events.onLoggedOut.addListener(this.onReset_.bind(this));
    Notification.Events.onChanged.addListener(this.onItemChanged_.bind(this));
    DataPlan.Events.onChanged.addListener(this.onDataPlanChanged_.bind(this));
    Proxy.Events.onEnabledChanged.addListener(
        this.onProxyAutoConnectChanged_.bind(this));
    ToolbarState.Events.onChanged.addListener(
        this.onProxyStateChanged_.bind(this));
  }

  initialize() {
    super.initialize();
    this.onProxyStateChanged_();
    this.onDataPlanChanged_();
  }

  get items() {
    return [].concat(this.items_);
  }

  get activeItems() {
    return this.items_.filter(function(item) {
      return item.isVisible;
    });
  }

  getItem(id) {
    return this.items_.filter(function(item) {
      return item.id === id;
    })[0];
  }

  isGroupActive(group) {
    return this.activeItems.some(function(item) {
      return item.group === group;
    });
  }

  hide(group) {
    this.activeItems.forEach(function(item) {
      if (item.group === group) {
        item.isVisible = false;
      }
    });
  }

  hideAll() {
    [
      Notifications.GROUP_CONNECTION,
      Notifications.GROUP_USAGE
    ].forEach(function(group) {
      chrome.notifications.clear(group);
    });
  }

  onItemChanged_(item) {
    if (item.id === Notification.ID_CONNECTED && item.isVisible) {
      settings.set(Notifications.CONNECTED_SHOWN, true);
    }
  }

  onReset_() {
    this.hideAll();
    this.isDataPlanOverBandwidth_ = false;
    this.isDataPlanExpired_ = false;
    settings.set(Notifications.CONNECTED_SHOWN, false);
  }

  onDataPlanChanged_() {
    if (!dataPlan.data || dataPlan.isUnlimited) {
      if (!this.isDataPlanExpired_ && dataPlan.isExpired && proxy.isEnabled) {
        this.isDataPlanExpired_ = dataPlan.isExpired;
        this.getItem(Notification.ID_EXPIRED).isVisible = true;
        Url.openMiniInternetTab(device.id, Url.PLAN_EXPIRED);
      } else {
        this.lastUsedDataPercent_ = null;
        this.hideAll();
      }

      return;
    }

    var percent = dataPlan.used / dataPlan.total;
    percent = Math.floor(percent * 100);

    var newPercent = 0;

    if (percent === 100) {
      newPercent = 100;
    } else if (percent >= 80) {
      newPercent = 80;
    } else if (percent >= 50) {
      newPercent = 50;
    } else if (percent >= 25) {
      newPercent = 25;
    }

    if (!this.isDataPlanOverBandwidth_ && dataPlan.isOverBandwidth &&
        proxy.isEnabled) {
      this.lastUsedDataPercent_ = 0;
    }

    this.isDataPlanOverBandwidth_ = dataPlan.isOverBandwidth;

    if (this.lastUsedDataPercent_ === null ||
        this.lastUsedDataPercent_ > newPercent) {
      this.lastUsedDataPercent_ = newPercent;
    } else if (this.lastUsedDataPercent_ !== newPercent) {
      this.lastUsedDataPercent_ = newPercent;
      this.showUsedNotification_(percent);
    }
  }

  onProxyAutoConnectChanged_() {
    if (!proxy.isEnabled) {
      this.onReset_();
    } else {
      this.onDataPlanChanged_();
    }
  }

  onProxyStateChanged_() {
    if (toolbarState.state === ToolbarState.STATE_CONNECTED) {
      if (!settings.get(Notifications.CONNECTED_SHOWN)) {
        this.getItem(Notification.ID_CONNECTED).isVisible = true;
      }
    } else {
      this.hide(Notifications.GROUP_CONNECTION);
    }
  }

  showUsedNotification_(percent) {
    this.hide(Notifications.GROUP_USAGE);
    if (percent === 100) {
      this.getItem(Notification.ID_OVER_BANDWIDTH).isVisible = true;
      Url.openMiniInternetTab(device.id, Url.PLAN_OVER_BANDWIDTH);
      return;
    }

    var item = this.getItem(Notification.ID_USED);
    item.title = i18n.get('notificationsUsageTitle', [percent + '%']);
    item.isVisible = true;
  }
};

Notifications.CONNECTED_SHOWN = 'notifications.connectedShown';
Notifications.GROUP_CONNECTION = 'surfeasy-connection';
Notifications.GROUP_USAGE = 'surfeasy-usage';

window.Notifications = Notifications;
