'use strict';

class Proxy extends Base {
  constructor() {
    super();
    settings.register(Proxy.SETTING_IS_ENABLED, true);
    settings.register(Proxy.SETTING_LEVEL_OF_CONTROL,
                      Proxy.CONTROLLABLE_BY_THIS_EXTENSION);
    settings.register(Proxy.SETTING_SERVER, null);
    settings.register(Proxy.SETTING_SERVER_PATTERN,
                      Url.PROXY_SERVER_PATTERN_DEFAULT);
    settings.register(Proxy.SETTING_BYPASS_LIST, []);

    this.canConnect_ = false;
    this.configFile_ = null;
    this.hasServerErrors_ = false;
    this.recivedServerErrors_ = false;

    this.serviceCheckTask_ = new NetworkTask(
        Proxy.SERVICE_CHECK_TASK_ID,
        this.onServiceCheckRequest_.bind(this),
        this.canRunServiceCheckTask_.bind(this));
    networkTasks.add(this.serviceCheckTask_);

    chrome.webRequest.onAuthRequired.addListener(
        this.onAuthRequired_.bind(this),
        {'urls': ['<all_urls>']}, ['asyncBlocking']);
  };

  registerListeners() {
    super.registerListeners();
    chrome.proxy.onProxyError.addListener(this.onError_.bind(this));
    chrome.proxy.settings.onChange.addListener(
        this.onProxySettingsChanged_.bind(this));

    Api.Events.onProxyServerPatternReceived.addListener(
        this.onServerPatternChanged_.bind(this));

    var onCanConnectChanged_ = this.onCanConnectChanged_.bind(this);
    Device.Events.onChanged.addListener(onCanConnectChanged_);
    Reward.Events.onChanged.addListener(onCanConnectChanged_);
    DataPlan.Events.onChanged.addListener(onCanConnectChanged_);
    Geo.Events.onChanged.addListener(onCanConnectChanged_);
    Errors.Events.onChanged.addListener(onCanConnectChanged_);
  }

  initialize() {
    super.initialize();
    chrome.proxy.settings.get({}, this.onProxySettingsChanged_.bind(this));
    this.onCanConnectChanged_();
    this.applyConfig_(false);
  }

  get isEnabled() {
    return settings.get(Proxy.SETTING_IS_ENABLED);
  }

  set isEnabled(value) {
    if (this.isEnabled === value) {
      return;
    }

    settings.set(Proxy.SETTING_IS_ENABLED, value);

    Proxy.Events.onEnabledChanged.dispatch(this);

    this.onCanConnectChanged_();
  }

  get levelOfControl() {
    return settings.get(Proxy.SETTING_LEVEL_OF_CONTROL);
  }

  set levelOfControl(value) {
    if (this.levelOfControl === value) {
      return;
    }

    settings.set(Proxy.SETTING_LEVEL_OF_CONTROL, value);

    Proxy.Events.onControllableChanged.dispatch(this);

    this.onCanConnectChanged_();
  }

  get isControlled() {
    return this.levelOfControl === Proxy.CONTROLLED_BY_THIS_EXTENSION;
  }

  get defaultServer() {
    return this.serverPattern.replace(
        '%@',
        Url.withStagingPostfix(Geo.DEFAULT_COUNTRY_CODE.toLowerCase()));
  }

  get server() {
    return settings.get(Proxy.SETTING_SERVER);
  }

  set server(value) {
    if (this.server === value) {
      return;
    }

    settings.set(Proxy.SETTING_SERVER, value);

    if (this.isConnected) {
      this.applyConfig_();
    }
  }

  get isConnected() {
    return this.canConnect && this.isControlled;
  }

  get canConnect() {
    return this.canConnect_;
  }

  get hasServerErrors() {
    return this.isEnabled && this.hasServerErrors_;
  }

  get serverPattern() {
    return settings.get(Proxy.SETTING_SERVER_PATTERN);
  }

  set serverPattern(value) {
    if (this.serverPattern === value) {
      return;
    }

    settings.set(Proxy.SETTING_SERVER_PATTERN, value);
    this.validateServer();
  }

  get bypassList() {
    return settings.get(Proxy.SETTING_BYPASS_LIST);
  }

  set bypassList(value) {
    settings.set(Proxy.SETTING_BYPASS_LIST, value);

    if (this.isConnected) {
      this.applyConfig_(false);
    }
  }

  applyConfig_(shouldForceLogin) {
    this.loadConfiguration().then(function(config) {
      chrome.proxy.settings.set(
          {
            'value': config,
            'scope': 'regular'
          },
          shouldForceLogin ? this.logout_.bind(this) : doNothing);
    }.bind(this)).catch(doNothing);
  }

  getConfig_() {
    return new Promise(function(resolve, reject) {
      if (this.configFile_) {
        resolve(this.configFile_);
      } else {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', Url.PROXY_PAC_SCRIPT);
        xhr.responseType = 'text';
        xhr.onerror = reject;
        xhr.onload = function() {
          this.configFile_ = xhr.response;
          resolve(this.configFile_);
        }.bind(this);
        xhr.send();
      };
    }.bind(this));
  }

  loadConfiguration() {
    this.validateServer();
    return this.getConfig_().then(function(file) {
      var bypassList = this.bypassList.map(function(value) {
        return '"' + value + '"';
      }).join(',');

      file = file.replace('$$HOST$$', Url.HOST);
      file = file.replace('$$HOST_PATTERN$$', Url.HOST_PATTERN);
      file = file.replace('$$IS_ENABLED$$', this.canConnect);
      file = file.replace('$$PROXY_SERVER$$',
          this.canConnect ? this.server : this.defaultServer);
      file = file.replace('$$TIMESTAMP$$', Date.now());
      file = file.replace('$$BYPASS_LIST$$', bypassList);

      return {
        'mode': 'pac_script',
        'pacScript': {
          'data': file
        }
      };
    }.bind(this));
  }

  validateServer() {
    this.server = this.serverPattern.replace('%@',
        Url.withStagingPostfix(geo.countryCode.toLowerCase()));
  }

  reloadLoadingTabs() {
    chrome.tabs.query({}, function(tabs) {
      if (!tabs || !tabs.length) {
        return;
      }
      tabs.filter(function (tab) {
        var isLoading = tab.status === undefined ||
            tab.status === chrome.tabs.TabStatus.LOADING;

        var isInternal =
            /(chrome-extension|chrome|browser)\:\/\/*/.test(tab.url);

        var isExtensionsPage =
            /(chrome|browser)\:\/\/extensions*/.test(tab.url);

        return !isExtensionsPage && (isLoading || isInternal);
      }).map(function(tab) {
        return tab.id;
      }).forEach(function(id) {
        chrome.tabs.reload(id);
      });
    });
  }

  onServerPatternChanged_(pattern) {
    this.serverPattern = pattern;
  }

  onProxySettingsChanged_(details) {
    this.levelOfControl = details.levelOfControl;
  }

  onCanConnectChanged_() {
    var canConnect = this.isEnabled && !errors.isActiveErrorCritical &&
        device.isConnected && rewards.areWelcomeRewardsCompleted &&
        Boolean(geo.list.length) && dataPlan.isValid;

    if (this.canConnect_ !== canConnect) {
      this.canConnect_ = canConnect;
      Proxy.Events.onCanConnectChanged.dispatch(this);

      this.applyConfig_(true);
    } else if (this.isConnected) {
      this.validateServer();
    }
  }

  onError_() {
    // todo('Check firewall and blocked ports');
    this.recivedServerErrors_ = true;
  }

  onServiceCheckRequest_(resolve, reject) {
    this.recivedServerErrors_ = false;
    api.serviceCheck().then(function(isWorking) {
      if (isWorking) {
        if (this.hasServerErrors_) {
          this.hasServerErrors_ = false;
          errors.reload();
        }
      } else if (!this.hasServerErrors_) {
        this.hasServerErrors_ = true;
        Proxy.Events.onError.dispatch(this);
      }
      resolve(new NetworkTaskData(isWorking, 3 * Api.TIMEOUT));
    }.bind(this)).catch(function() {
      if (!this.hasServerErrors_) {
        this.hasServerErrors_ = true;
        Proxy.Events.onError.dispatch(this);
      }
      resolve(new NetworkTaskData(false, 3 * Api.TIMEOUT));
    }.bind(this));
  }

  canRunServiceCheckTask_() {
    return device.isConnected && this.isControlled && this.recivedServerErrors_;
  }

  onAuthRequired_(details, callback) {
    var response = {};

    if (details.isProxy && details.challenger &&
        this.matchServerPattern_(details.challenger.host) && device.isValid) {
      response.authCredentials = {
        username: hash(device.id),
        password: device.password
      };
    }

    callback(response);

    if (details.url === this.getServerLogoutURL_() &&
        this.logoutPromiseResolve_) {
      this.logoutPromiseResolve_();
    }
  }

  matchServerPattern_(server) {
    return server.endsWith(this.serverPattern.replace('%@', ''));
  }

  getServerLogoutURL_() {
    if (this.server) {
      return Url.HTTP + [this.server, 'logout'].join('/');
    }
  }

  logout_() {
    var url = this.getServerLogoutURL_();

    var onLoggedOut = new Promise(function(resolve, reject) {
      this.logoutPromiseResolve_ = resolve;

      if (api.isOnLine && url) {
        try {
          var xhr = new XMLHttpRequest();
          xhr.open('GET', url);
          xhr.withCredentials = true;
          xhr.timeout = Api.TIMEOUT;
          xhr.onerror = xhr.ontimeout = xhr.onloadend = function () {
            resolve();
          };

          onLoggedOut.then(function() {
            xhr.abort();
          });

          xhr.send();
        } catch(e) {
          resolve();
        }
      } else {
        resolve();
      }
    }.bind(this));

    onLoggedOut.then(this.reloadLoadingTabs.bind(this));
  }
};

Proxy.SETTING_IS_ENABLED = 'proxy.autoConnect';
Proxy.SETTING_LEVEL_OF_CONTROL = 'proxy.levelOfControl';
Proxy.SETTING_SERVER = 'proxy.server';
Proxy.SETTING_SERVER_PATTERN = 'proxy.serverPattern';
Proxy.SETTING_BYPASS_LIST = 'proxy.bypassList';

Proxy.SERVICE_CHECK_TASK_ID = 'serviceCheck';

Proxy.CONTROLLABLE_NOT = 'not_controllable';
Proxy.CONTROLLED_BY_OTHER_EXTENSION = 'controlled_by_other_extensions';
Proxy.CONTROLLABLE_BY_THIS_EXTENSION = 'controllable_by_this_extension';
Proxy.CONTROLLED_BY_THIS_EXTENSION = 'controlled_by_this_extension';

Proxy.Events = {
  onCanConnectChanged: new chrome.Event(),
  onEnabledChanged: new chrome.Event(),
  onControllableChanged: new chrome.Event(),
  onStateChanged: new chrome.Event(),
  onError: new chrome.Event()
};

window.Proxy = Proxy;
