'use strict';

class Rewards extends Base {
  constructor() {
    super();
    this.items_ = new Set();
    this.items_.add(new Reward({
      'isLocal': true,
      'code': Reward.REWARD_WELCOME,
      'amount': Reward.REWARD_WELCOME_AMOUNT,
      'completed': true,
      'menu_title': i18n.get('rewardWelcomeDescription',
          [Reward.REWARD_WELCOME_AMOUNT]),
      'icon': null,
      'url': null
    }));
    this.items_.add(new Reward({
      'isLocal': true,
      'code': Reward.REWARD_NUMBER_INPUT,
      'amount': Reward.REWARD_NUMBER_INPUT_AMOUNT,
      'completed': true,
      'menu_title': i18n.get('rewardNumberInputDescription',
          [Reward.REWARD_NUMBER_INPUT_AMOUNT]),
      'icon': null,
      'url': null
    }));

    this.lastDataAmount_ = 0;
    this.task_ = new NetworkTask(Rewards.TASK_ID, this.onRequest_.bind(this),
                                 this.canRunTask_.bind(this));
    networkTasks.add(this.task_);
  }

  registerListeners() {
    super.registerListeners();

    User.Events.onRegistered.addListener(this.onUserRegistered_.bind(this));
    User.Events.onLoggedIn.addListener(this.onUserLoggedIn_.bind(this));
    User.Events.onLoggedOut.addListener(this.onUserLoggedOut_.bind(this));
    DataPlan.Events.onChanged.addListener(this.onDataPlanChanged_.bind(this));
    NetworkTask.Events.onData.addListener(
        this.onNetworkDataChanged_.bind(this));
  }

  get items() {
     var items = [];

    this.items_.forEach(function(reward) {
      items.push(reward);
    });

    return items;
  }

  get hasActiveItems() {
    return Boolean(this.activeItems.length);
  }

  get activeItems() {
    return rewards.items.filter(function(item) {
      return !item.isLocal && !item.isCompleted && Boolean(item.url);
    });
  }

  get areWelcomeRewardsCompleted() {
    var welcomeReward = this.getItem(Reward.REWARD_WELCOME);
    var numberInputReward = this.getItem(Reward.REWARD_NUMBER_INPUT);

    return (!welcomeReward || welcomeReward.isCompleted) &&
        (!numberInputReward || numberInputReward.isCompleted);
  }

  set areWelcomeRewardsCompleted(value) {
    var welcomeReward = this.getItem(Reward.REWARD_WELCOME);
    var numberInputReward = this.getItem(Reward.REWARD_NUMBER_INPUT);
    if (welcomeReward) {
      welcomeReward.isCompleted = value;
    }

    if (numberInputReward) {
      numberInputReward.isCompleted = value;
    }
  }

  getItem(code) {
    return this.items.reduce(function(current, reward) {
      return reward.code === code ? reward : current;
    }, null);
  }

  canRunTask_() {
    return user.isLoggedIn;
  }

  onRequest_(resolve, reject) {
    api.rewards().then(function(rewards) {
      resolve(new NetworkTaskData(rewards, 10 * 60 * 1000));
    }).catch(reject);
  }

  clear() {
    this.items_.forEach(function(reward) {
      if (!reward.isLocal) {
        this.items_.delete(reward);
      }
    }.bind(this));
    Rewards.Events.onChanged.dispatch(this);
  }

  onUserRegistered_() {
    this.areWelcomeRewardsCompleted = false;
  }

  onUserLoggedIn_() {
    if (user.isLoggedIn) {
      this.task_.run();
    }
  }

  onUserLoggedOut_() {
    this.areWelcomeRewardsCompleted = true;
    this.task_.clear();
  }

  onDataPlanChanged_() {
    if (dataPlan.isRewardable && dataPlan.total) {
      if (this.lastDataAmount_ !== 0 &&
          this.lastDataAmount_ !== dataPlan.total) {
        this.task_.run();
      }

      this.lastDataAmount_ = dataPlan.total;
    }
  }

  onNetworkDataChanged_(item) {
    if (item.name === Rewards.TASK_ID) {
      this.clear();
      if (Array.isArray(item.data)) {
        item.data.forEach(function (data) {
          this.items_.add(new Reward(data));
        }.bind(this));
        Rewards.Events.onChanged.dispatch(this);
      }
    }
  }
};

Rewards.Events = {
  onChanged: new chrome.Event()
};

Rewards.TASK_ID = 'rewards';

window.Rewards = Rewards;
