'use strict';

class SessionCookie extends Base {
  constructor() {
    super();
    settings.register(SessionCookie.SETTING_LAST, null);
    this.isStorable_ = false;

    this.restoreIfSessionOnly_();
  }

  registerListeners() {
    super.registerListeners();

    chrome.cookies.onChanged.addListener(this.onUpdated_.bind(this));
  }

  initialize() {
    super.initialize();
    this.check().then(doNothing).catch(doNothing);
  }

  get last() {
    return settings.get(SessionCookie.SETTING_LAST);
  }

  set last(value) {
    if (JSON.stringify(this.last) === JSON.stringify(value)) {
      return;
    }

    settings.set(SessionCookie.SETTING_LAST, value);
  }

  get isStorable() {
    return this.isStorable_;
  }

  set isStorable(value) {
    if (this.isStorable_ === value) {
      return;
    }

    this.isStorable_ = value;
    SessionCookie.Events.onChanged.dispatch(this);
  }

  addException() {
    chrome.contentSettings.cookies.set({
      'primaryPattern': Url.HTTPS + '*.' + Url.HOST + '/*',
      'setting': chrome.contentSettings.CookiesContentSetting.ALLOW
    }, function() {
      this.check().catch(doNothing);
    }.bind(this));
  }

  remove() {
    return new Promise(function(resolve, reject) {
      chrome.cookies.remove({
        'url': Url.BASE,
        'name': SessionCookie.NAME
      }, function() {
        this.last = null;
        resolve();
      }.bind(this));
    }.bind(this));
  }

  restoreIfSessionOnly_() {
    if (!this.last) {
      return;
    }

    var cookie = this.last;

    chrome.contentSettings.cookies.get({
      'primaryUrl': 'ftp://*/*'
    }, function(details) {
      if (!details || !details.setting) {
        return;
      }
      if (details.setting === SessionCookie.SESSION_ONLY) {
        chrome.cookies.set({
          'url': Url.BASE,
          'name': cookie.name,
          'value': cookie.value,
          'domain': cookie.domain,
          'path': cookie.path,
          'secure': cookie.secure,
          'httpOnly': cookie.httpOnly,
          'expirationDate': cookie.expirationDate
        });
      }
    }.bind(this));
  }

  check() {
    return new Promise(function(resolve, reject) {
      chrome.contentSettings.cookies.get({
        'primaryUrl': Url.API + '/*'
      }, function(details) {
        if (details.setting === SessionCookie.ALLOW) {
          this.isStorable = true;
          resolve();
        } else {
          this.isStorable = false;
          reject();
        }
      }.bind(this));
    }.bind(this));
  }

  onUpdated_(details) {
    if (!details || details.cause !== chrome.cookies.OnChangedCause.OVERWRITE ||
        !details.cookie) {
      return;
    }

    var cookie = details.cookie;
    if (cookie.name !== SessionCookie.NAME ||
        !cookie.domain.endsWith(Url.HOST)) {
      return;
    }

    this.last = cookie;
  }
}

SessionCookie.NAME = 'api_session';
SessionCookie.SETTING_LAST = 'sessionCookie.last';
SessionCookie.ALLOW = chrome.contentSettings.CookiesContentSetting.ALLOW;
SessionCookie.BLOCK = chrome.contentSettings.CookiesContentSetting.BLOCK;
SessionCookie.SESSION_ONLY =
    chrome.contentSettings.CookiesContentSetting.SESSION_ONLY;

SessionCookie.Events = {
  onChanged: new chrome.Event()
};

window.SessionCookie = SessionCookie;
