'use strict';

class ToolbarState extends Base {
  constructor() {
    super();
    chrome.browserAction.setPopup({
      'popup': Url.TOOLBAR_POPUP
    });
    this.loadingResources_ = [];
    this.setIconPromise_ = Promise.resolve();
    this.state = ToolbarState.STATE_DISCONNECTED;
    this.badgeState = ToolbarState.BADGE_STATE_INACTIVE;
  }

  registerListeners() {
    super.registerListeners();

    var onStateChanged_ = this.onStateChanged_.bind(this);
    User.Events.onChanged.addListener(onStateChanged_);
    Reward.Events.onChanged.addListener(onStateChanged_);
    DataPlan.Events.onChanged.addListener(onStateChanged_);
    Geo.Events.onChanged.addListener(onStateChanged_);
    Proxy.Events.onCanConnectChanged.addListener(onStateChanged_);
    Proxy.Events.onControllableChanged.addListener(onStateChanged_);
    Errors.Events.onChanged.addListener(onStateChanged_);

    Errors.Events.onChanged.addListener(this.onBadgeStateChanged_.bind(this));
  }

  initialize() {
    super.initialize();
    this.onStateChanged_();
    this.onBadgeStateChanged_();
  }

  get badgeState() {
    return this.badgeState_;
  }

  set badgeState(value) {
    if (this.badgeState_ === value) {
      return;
    }
    this.badgeState_ = value;

    var badgeText = this.badgeState_ === ToolbarState.BADGE_STATE_ACTIVE ?
        ToolbarState.BADGE_TEXT : '';

    chrome.browserAction.setBadgeText({
      'text': badgeText
    });
  }

  get state() {
    return this.state_;
  }

  set state(value) {
    if (this.state_ === value) {
      return;
    }

    var icons = ToolbarState.Icons[value];

    if (!icons) {
      return;
    }

    this.state_ = value;
    this.setIconPromise_ = new Promise(function(queue, resolve, reject) {
      queue.then(function() {
        chrome.browserAction.setIcon({
          'path': icons
        }, resolve);
      });
    }.bind(this, this.setIconPromise_));

    ToolbarState.Events.onChanged.dispatch(this);
  }

  setLoading(id, shouldLoading) {
    var index = this.loadingResources_.indexOf(id);
    var isLoading = index !== -1;

    if (shouldLoading && !isLoading) {
      this.loadingResources_.push(id);
    } else if (!shouldLoading && isLoading) {
      this.loadingResources_.splice(index, 1);
    }

    this.onStateChanged_();
  }

  onStateChanged_() {
    var isReadyForConnection = user.isLoggedIn &&
        rewards.areWelcomeRewardsCompleted && !errors.hasActive;
    var isProxyConnecting = proxy.canConnect && !proxy.isConnected;
    var isGettingDataPlan = isReadyForConnection && !Boolean(dataPlan.data);
    var isGettingGeo = isReadyForConnection && (!Boolean(geo.list.length) ||
        !(Boolean(geo.userLocation) || geo.userLocationUnknown) ||
        (proxy.isConnected && !Boolean(geo.serverLocation)));

    if (this.loadingResources_.length || isGettingDataPlan ||
        isGettingGeo || isProxyConnecting) {
      this.state = ToolbarState.STATE_CONNECTING;
    } else if (proxy.isConnected) {
      this.state = ToolbarState.STATE_CONNECTED;
    } else {
      this.state = ToolbarState.STATE_DISCONNECTED;
    }
  }

  onBadgeStateChanged_() {
    this.badgeState = errors.hasActive;
  }
};

ToolbarState.STATE_DISCONNECTED = 0;
ToolbarState.STATE_CONNECTING = 1;
ToolbarState.STATE_CONNECTED = 2;

ToolbarState.BADGE_STATE_ACTIVE = true;
ToolbarState.BADGE_STATE_INACTIVE = false;

ToolbarState.BADGE_TEXT = '1';

ToolbarState.Events = {
  onChanged: new chrome.Event()
};

ToolbarState.Icons = [
  {
    '19': '/images/toolbar_icon/disconnected.png',
    '38': '/images/toolbar_icon/2x/disconnected.png'
  },
  {
    '19': '/images/toolbar_icon/connecting.png',
    '38': '/images/toolbar_icon/2x/connecting.png'
  },
  {
    '19': '/images/toolbar_icon/connected.png',
    '38': '/images/toolbar_icon/2x/connected.png'
  }
];

window.ToolbarState = ToolbarState;
