'use strict';

class User extends Base {
  constructor() {
    super();
    settings.register(User.SETTING_EMAIL);
    settings.register(User.SETTING_IS_LOGGED, false);
    settings.register(User.SETTING_CAN_REGISTER, true);

    this.isAutoLogin_ = this.isLoggedIn;
  }

  registerListeners() {
    super.registerListeners();

    Api.Events.onUserError.addListener(this.logout.bind(this));
  }

  initialize() {
    super.initialize();
    this.onChanged_();

    if (this.isLoggedIn) {
      User.Events.onLoggedIn.dispatch(this);
    }
  }

  get isAutoLogin() {
    return this.isAutoLogin_;
  }

  get canRegister() {
    return settings.get(User.SETTING_CAN_REGISTER);
  }

  set canRegister(value) {
    if (this.canRegister === value) {
      return;
    }

    settings.set(User.SETTING_CAN_REGISTER, value);
    User.Events.onCanRegisterChanged.dispatch(this);
  }

  get isValid() {
    return Boolean(this.email) && this.isLoggedIn;
  }

  get isLoggedIn() {
    return settings.get(User.SETTING_IS_LOGGED);
  }

  set isLoggedIn(value) {
    if (this.isLoggedIn === value) {
      return;
    }

    settings.set(User.SETTING_IS_LOGGED, value);
    if (!value) {
      this.isAutoLogin_ = false;
    }

    this.onChanged_();

    if (!this.isLoggedIn) {
      User.Events.onLoggedOut.dispatch(this);
    } else {
      User.Events.onLoggedIn.dispatch(this);
    }
  }

  get email() {
    return settings.get(User.SETTING_EMAIL);
  }

  set email(value) {
    if (this.email === value) {
      return;
    }

    settings.set(User.SETTING_EMAIL, value);
    this.isLoggedIn = false;
    this.onChanged_();
  }

  register(email, password) {
    return new Promise(function(resolve, reject) {
      if (!email || !password) {
        reject();
      } else {
        toolbarState.setLoading(User.ID_REGISTER, true);
        this.email = email;
        api.register(this.email, hash(password)).then(function() {
          this.canRegister = false;
          User.Events.onRegistered.dispatch(this);
          this.isLoggedIn = true;
          resolve();
          toolbarState.setLoading(User.ID_REGISTER, false);
        }.bind(this)).catch(function(details) {
          this.email = null;
          reject(details);
          toolbarState.setLoading(User.ID_REGISTER, false);
        }.bind(this));
      }
    }.bind(this));
  }

  login(email, password) {
    return new Promise(function(resolve, reject) {
      if (!email || !password) {
        reject();
      } else if (this.isValid) {
        resolve();
      } else {
        toolbarState.setLoading(User.ID_LOGIN, true);
        this.email = email;
        api.login(this.email, hash(password)).then(function() {
          this.canRegister = false;
          this.isLoggedIn = true;
          resolve();
          toolbarState.setLoading(User.ID_LOGIN, false);
        }.bind(this)).catch(function(details) {
          this.email = null;
          reject(details);
          toolbarState.setLoading(User.ID_LOGIN, false);
        }.bind(this));
      }
    }.bind(this));
  }

  logout() {
    this.isLoggedIn = false;
  }

  getAutoLoginToken() {
    return new Promise(function(resolve, reject) {
      if (!this.isLoggedIn) {
        reject();
      }

      api.getAutoLoginToken().then(function(data) {
        if (data.auto_login_token) {
          resolve(data.auto_login_token);
        } else {
          reject();
        }
      }.bind(this)).catch(reject);
    }.bind(this));
  }

  onChanged_() {
    User.Events.onChanged.dispatch(this);
  }

};

User.Events = {
  onLoggedIn: new chrome.Event(),
  onLoggedOut: new chrome.Event(),
  onRegistered: new chrome.Event(),
  onCanRegisterChanged: new chrome.Event(),
  onChanged: new chrome.Event()
};

User.ID_LOGIN = 'user.login';
User.ID_REGISTER = 'user.register';
User.SETTING_EMAIL = 'user.email';
User.SETTING_IS_LOGGED = 'user.isLogged';
User.SETTING_CAN_REGISTER = 'user.canRegister';

window.User = User;
