'use strict';

class AbstractForm extends AbstractPage {
  constructor(id) {
    super(id);
  }

  get form() {
    return this.viewElement.querySelector('form');
  }

  get isValid() {
    return every(this.form.elements, function(element) {
      return element.checkValidity();
    });
  }

  registerHandlers_() {
    super.registerHandlers_();
    var onFormElementChanged_ = bindOnce(this.onFormElementChanged_, this);
    EventHandler.register('keydown', AbstractForm.HANDLER_FORM_ELEMENT_CHANGE,
                          onFormElementChanged_);
    EventHandler.register('change', AbstractForm.HANDLER_FORM_ELEMENT_CHANGE,
                          onFormElementChanged_);
    EventHandler.register('submit', AbstractForm.HANDLER_FORM_SUBMIT,
                          bindOnce(this.onFormSubmited_, this));
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();
    var onFormElementChanged_ = bindOnce(this.onFormElementChanged_, this);
    EventHandler.unregister('keydown', AbstractForm.HANDLER_FORM_ELEMENT_CHANGE,
                            onFormElementChanged_);
    EventHandler.unregister('change', AbstractForm.HANDLER_FORM_ELEMENT_CHANGE,
                            onFormElementChanged_);
    EventHandler.unregister('submit', AbstractForm.HANDLER_FORM_SUBMIT,
                            bindOnce(this.onFormSubmited_, this));
  }

  onFormElementChanged_(event, target) {
    target.classList.toggle(AbstractForm.CLASS_EMPTY, !target.value.length);
  }

  onFormSubmited_(event, target) {
    this.form.classList.add(AbstractForm.CLASS_FORM_VALIDATED);
    for_each(this.form.querySelectorAll(classesToSelector([
          AbstractForm.CLASS_FORM_ERROR,
          AbstractForm.CLASS_FORM_ELEMENT_ERROR
        ])),
        function(error) {
      error.textContent = '';
    });

    if (!this.isValid) {
      var element = this.form.querySelector(':invalid');
      if (element) {
        element.focus();
      }
    }

    event.preventDefault();
  }

  addFormElementError_(element, message) {
    if (!element) {
      return;
    }

    var formElement = element.closest(classesToSelector(
        AbstractForm.CLASS_FORM_ELEMENT));

    if (!formElement) {
      return;
    }

    var errorElement = formElement.querySelector(classesToSelector(
        AbstractForm.CLASS_FORM_ELEMENT_ERROR));
    if (!errorElement) {
      return;
    }

    errorElement.textContent = message;
  }

  addFormError(details) {
    if (!this.form || !details) {
      return;
    }

    var formError = this.form.querySelector(classesToSelector(
        AbstractForm.CLASS_FORM_ERROR));
    if (!formError) {
      return;
    }

    var messages = [];
    for (let code in details) {
      messages.push(this.Templates.formErrorMessage(details[code]));
    }
    renderTemplateIfNeeded(formError, messages);
  }
};

AbstractForm.Templates = class {
  static form(data) {
    data = data || {};
    data['novalidate'] = data['novalidate'] || '';
    data['data-handler'] = AbstractForm.HANDLER_FORM_SUBMIT;

    return ['form', data, this.formError()];
  }

  static formErrorMessage(text) {
    return [
      'div',
      text
    ];
  }

  static formError() {
    return [
      'p', {
        'class': AbstractForm.CLASS_FORM_ERROR
      }
    ];
  }

  static formElement(data, label, hint) {
    data = data || {};
    if (!data.value || !data.value.length) {
      data.class = [AbstractForm.CLASS_EMPTY, data.class].join(' ').trim();
    }
    if (data.required === undefined || data.required === true) {
      data.required = '';
    }
    data.type = data.type || AbstractForm.DEFAULT_FORM_ELEMENT_TYPE;
    data['data-handler'] = AbstractForm.HANDLER_FORM_ELEMENT_CHANGE;

    return [
      'div', {
        'class': AbstractForm.CLASS_FORM_ELEMENT
      },
      ['input', data],
      (label ? ['label', label] : null),
      ['span', {
        'class': AbstractForm.CLASS_FORM_ELEMENT_ERROR
      }],
      (hint ? [
            'span', {
              'class': AbstractForm.CLASS_FORM_ELEMENT_HINT
            },
            hint
          ] : null)
    ];
  }
};

AbstractForm.CLASS_EMPTY = 'empty';
AbstractForm.CLASS_FORM_VALIDATED = 'form-validated';
AbstractForm.CLASS_FORM_ERROR = 'form-error';
AbstractForm.CLASS_FORM_ELEMENT = 'form-element';
AbstractForm.CLASS_FORM_ELEMENT_ERROR = 'form-element-error';
AbstractForm.CLASS_FORM_ELEMENT_HINT = 'form-element-hint';

AbstractForm.DEFAULT_FORM_ELEMENT_TYPE = 'text';

AbstractForm.HANDLER_FORM_ELEMENT_CHANGE = 'form-element-change';
AbstractForm.HANDLER_FORM_SUBMIT = 'form-submit';
