'use strict';

class AbstractPageView extends AbstractView {
  constructor(id) {
    super(id);
    this.currentPageID_ = this.defaultPageID;
  }

  get title() {
    var title = '';

    if (this.title_) {
      title += this.title_;
    }

    if (this.currentPageID) {
      if (title && this.currentPage.title) {
        title += ' - ';
      }

      title += this.currentPage.title;
    }

    return title;
  }

  set title(value) {
    this.title_ = value;
  }

  get isVisible() {
    return super.isVisible;
  }

  set isVisible(value) {
    if (this.isVisible === value) {
      return;
    }

    super.isVisible = value;

    if (this.currentPage && !value) {
      this.currentPage.hide();
    }

    AbstractPageView.Events.onPageChanged.dispatch(this);
  }

  get isPageVisible() {
    return Boolean(this.currentPageID_);
  }

  get defaultPageID() {
    return null;
  }

  get currentPageID() {
    return this.currentPageID_;
  }

  set currentPageID(value) {
    if (this.currentPageID_ === value) {
      return;
    }

    if (this.currentPage && this.currentPage.isVisible) {
      this.currentPage.hide();
    }

    this.currentPageID_ = value;

    if (this.isVisible) {
      this.render();
      AbstractView.Events.onTitleChanged.dispatch(this);
      AbstractPageView.Events.onPageChanged.dispatch(this);
    }
  }

  get currentPage() {
    return site.getPage(this.currentPageID_);
  }

  renderSubpage(template) {
    var element = bragi.createTemplate(template);


    if (this.isPageVisible) {
      this.attachSubpages_(element, [this.currentPageID]);
    }

    if (areElementsDifferent(this.viewElement, element)) {
      this.viewElement.textContent = '';
      this.viewElement.appendChild(element);
    }
  }

  getPageContainers(element) {
    var pages = map(
        element.querySelectorAll(classesToSelector(
            AbstractPageView.CLASS_NAME_PAGE)),
        function(page) {
          return page;
        });
    return pages.length ? pages : [element];
  }

  attachSubpages_(element, subpages) {
    var pages = this.getPageContainers(element);

    pages.forEach(function(page, index) {
      var subpage = site.getPage(subpages[index]);
      if (subpage) {
        subpage.show(page);
      }
    });
  }

  registerHandlers_() {
    super.registerHandlers_();
    AbstractPage.Events.onVisibilityChanged.addListener(
        bindOnce(this.onPageVisibilityChanged_, this));
    EventHandler.register('click', AbstractPageView.HANDLER_NAVIGATE_TO_PAGE,
                          bindOnce(this.onNavigateToPage_, this));
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();
    AbstractPage.Events.onVisibilityChanged.addListener(
        bindOnce(this.onPageVisibilityChanged_, this));
    EventHandler.unregister('click', AbstractPageView.HANDLER_NAVIGATE_TO_PAGE,
                            bindOnce(this.onNavigateToPage_, this));
  }

  registerListeners_() {
    super.registerListeners_();
  }

  unregisterListeners_() {
    super.unregisterListeners_();
  }

  onNavigateToPage_(event, target) {
    var pageId = target.dataset.page;
    if (!pageId) {
      this.currentPageID = this.defaultPageID;
      return;
    }

    var page = site.getPage(pageId);
    if (!page) {
      this.currentPageID = this.defaultPageID;
      return;
    }

    this.currentPageID = pageId;
  }

  onPageVisibilityChanged_(page) {
    if (!this.isVisible) {
      return;
    }

    if (page.id === this.currentPageID && !page.isVisible) {
      this.currentPageID = this.defaultPageID;
    }
  }

};

AbstractPageView.Events = {
  onPageChanged: new chrome.Event()
};

AbstractPageView.Templates = class {
  static page() {
    return ['div', {'class': AbstractPageView.CLASS_NAME_PAGE}];
  }

  static header(isPageVisible) {
    var header = [];

    if (!isPageVisible) {
      header.push([
        'div', {
          'class': 'button-menu',
          'data-handler': AbstractPageView.HANDLER_NAVIGATE_TO_PAGE,
          'data-page': SideMenuPage.ID
        }
      ]);
    } else {
      header.push([
        'div', {
          'class': 'button-back',
          'data-handler': AbstractPageView.HANDLER_NAVIGATE_TO_PAGE,
          'data-page': ''
        }
      ]);
    }

    return header;
  }
};

AbstractPageView.CLASS_NAME_PAGE = 'page';
AbstractPageView.CLASS_IS_PAGE_VISIBLE = 'is-page-visible';
AbstractPageView.HANDLER_NAVIGATE_TO_PAGE =
    'abstract-page-view-navigate-to-page';
