'use strict';

class RegisterForm extends AbstractForm {
  constructor() {
    super(RegisterForm.ID);
    this.title_= i18n.get('signInUpTitleRegister');
    this.cachedData_ = {
      'email': '',
      'password': '',
      'confirmPassword': ''
    };

    this.registerListeners_();
  }

  registerListeners_() {
    super.registerListeners_();

    User.Events.onRegistered.addListener(
        bindOnce(this.onUserRegistered_, this));
  }

  unregisterListeners_() {
    super.unregisterListeners_();

    User.Events.onRegistered.removeListener(
        bindOnce(this.onUserRegistered_, this));
  }

  get arePasswordsMatch() {
    return this.data.password === this.data.confirmPassword;
  }

  get isPasswordLongEnough() {
    return this.data.password.length >= RegisterForm.MIN_PASSWORD_LENGTH;
  }

  get isPasswordNotContainsOnlyRepeatingChars() {
    var password = this.data.password;
    var firstChar = password.charAt(0);

    return password.length && password.split('').some(function(char) {
          return char !== firstChar;
        });
  }

  get isValid() {
    return super.isValid && this.arePasswordsMatch &&
        this.isPasswordLongEnough &&
        this.isPasswordNotContainsOnlyRepeatingChars;
  }

  get data() {
    return {
      'email': this.form.email.value,
      'password': this.form.password.value,
      'confirmPassword': this.form.confirmPassword.value
    };
  }

  onFormSubmited_(event, target) {
    super.onFormSubmited_(event, target);

    this.cachedData_ = this.data;

    if (this.isValid) {
      RegisterForm.Events.onSubmited.dispatch(this.data);
    } else {
      if (this.form.email.value.length && !this.form.email.checkValidity()) {
        this.addFormElementError_(this.form.email,
            i18n.get('signInUpErrorEmailInvalid'));
      }

      if (!this.isPasswordLongEnough) {
        this.addFormElementError_(this.form.password,
            i18n.get('signInUpErrorPasswordTooShort',
                [RegisterForm.MIN_PASSWORD_LENGTH]));
      } else if (!this.isPasswordNotContainsOnlyRepeatingChars) {
        this.addFormElementError_(this.form.password,
            i18n.get('signInUpErrorPasswordOnlyRepeatingChars'));
      }

      if (!this.arePasswordsMatch) {
        this.addFormElementError_(this.form.confirmPassword,
            i18n.get('signInUpErrorConfirmPassword'));
      }
    }
  }

  onUserRegistered_() {
    this.cachedData_ = {
      'email': '',
      'password': '',
      'confirmPassword': ''
    };
  }

  render() {
    renderTemplateIfNeeded(this.viewElement,
        this.Templates.view(this.cachedData_));
  }
};

RegisterForm.Templates = class extends AbstractForm.Templates {
  static header() {
    return [
      'h1',
      ['strong', i18n.get('signInUpTitle')],
      ['span', i18n.get('signInUpTitleRegister')]
    ];
  }

  static linkGoLogin() {
    return [
      'span', {
        'class': 'link',
        'tabindex': '0',
        'data-handler': AbstractPageView.HANDLER_NAVIGATE_TO_PAGE,
        'data-page': LoginForm.ID
      },
      i18n.get('signInUpLinkGoToLogin')
    ];
  }

  static buttonSubmit() {
    return ['button', i18n.get('buttonRegister')];
  }

  static summary() {
    return [
      'div', {
        'class': 'summary'
      },
      this.linkGoLogin(),
      this.buttonSubmit()
    ];
  }

  static form(data) {
    var form = super.form();

    form.push(this.formElement({
          'type': 'email',
          'name': 'email',
          'value':  data.email
        }, i18n.get('labelEmail')));
    form.push(this.formElement({
          'type': 'password',
          'name': 'password',
          'value':  data.password
        }, i18n.get('labelPassword')));
    form.push(this.formElement({
          'type': 'password',
          'name': 'confirmPassword',
          'value':  data.confirmPassword
        }, i18n.get('labelConfirmPassword')));

    form.push(this.summary());

    return form;
  }

  static view(data) {
    return [
      this.header(),
      this.form(data)
    ];
  }
};

RegisterForm.Events = {
  onSubmited: new chrome.Event()
};

RegisterForm.ID = 'register';

RegisterForm.MIN_PASSWORD_LENGTH = 4;
