'use strict';

class AbstractHomeView extends AbstractPageView {
  constructor(id) {
    super(id);

    this.headerType = AbstractView.HEADER_NORMAL;
    this.viewElement.classList.add(AbstractHomeView.CLASS_NAME);

    this.homePages_ = [
      DisconnectedPlanPage.ID,
      JustConnectedPlanPage.ID
    ];
  }

  registerHandlers_() {
    super.registerHandlers_();
    EventHandler.register('click', AbstractHomeView.HANDLER_TOGGLE_CONNECT,
                          bindOnce(this.onToogleConnect_, this));
    LocationPage.Events.onSelected.addListener(
        bindOnce(this.onLocationSelected_, this));
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();
    EventHandler.unregister('click', AbstractHomeView.HANDLER_TOGGLE_CONNECT,
                            bindOnce(this.onToogleConnect_, this));
    LocationPage.Events.onSelected.removeListener(
        bindOnce(this.onLocationSelected_, this));
  }

  registerListeners_() {
    super.registerListeners_();
    DataPlan.Events.onChanged.addListener(
        bindOnce(this.onCanShowChanged_, this));

    Proxy.Events.onControllableChanged.addListener(
        bindOnce(this.onProxyChanged_, this));
    Proxy.Events.onEnabledChanged.addListener(
        bindOnce(this.onProxyChanged_, this));
    Proxy.Events.onCanConnectChanged.addListener(
        bindOnce(this.onProxyChanged_, this));
    Geo.Events.onChanged.addListener(bindOnce(this.onChanged_, this));
    Notification.Events.onChanged.addListener(
        bindOnce(this.onNotificationChanged_, this));
  }

  unregisterListeners_() {
    super.unregisterListeners_();
    DataPlan.Events.onChanged.removeListener(
        bindOnce(this.onCanShowChanged_, this));

    Proxy.Events.onControllableChanged.removeListener(
        bindOnce(this.onProxyChanged_, this));
    Proxy.Events.onEnabledChanged.removeListener(
        bindOnce(this.onProxyChanged_, this));
    Proxy.Events.onCanConnectChanged.removeListener(
        bindOnce(this.onProxyChanged_, this));
    Geo.Events.onChanged.removeListener(bindOnce(this.onChanged_, this));
    Notification.Events.onChanged.removeListener(
        bindOnce(this.onNotificationChanged_, this));
  }

  get isJustConnected() {
    return proxy.isConnected &&
        notifications.isGroupActive(Notifications.GROUP_CONNECTION);
  }

  get defaultHomePageID() {
    return DisconnectedPlanPage.ID;
  }

  get defaultPageID() {
    return this.isJustConnected ? JustConnectedPlanPage.ID :
        this.defaultHomePageID;
  }

  get isHomePageVisible() {
    return this.homePages_.indexOf(this.currentPageID) !== -1;
  }

  get isShowable() {
    return Boolean(dataPlan.data);
  }

  get templateData_() {
    var serverLocation = '';
    var serverIP = '';
    if (proxy.isConnected && geo.countryName) {
      serverLocation = geo.countryName;
      serverIP = geo.serverLocation.ip;
    } else if (geo.userLocation) {
      var city = geo.userLocation.city_name;
      var country = geo.userLocation.country_name;
      serverLocation = city + (city ? ', ' : '') + country;
      serverIP = geo.userLocation.ip;
    }

    return {
      'isConnected': proxy.isConnected,
      'server': {
        'ip': serverIP,
        'location': serverLocation
      },
      'hasUserIP': !geo.userLocationUnknown
    };
  }

  renderHeader_() {
    this.headerType = proxy.isConnected ? AbstractView.HEADER_NORMAL :
        AbstractView.HEADER_ERORR;

    this.headerElement.classList.toggle(AbstractPageView.CLASS_IS_PAGE_VISIBLE,
                                        !this.isHomePageVisible);

    renderTemplateIfNeeded(this.headerElement,
                           this.Templates.header(this.isHomePageVisible,
                                                 this.currentPageID));
  }

  render() {
    if (this.isHomePageVisible) {
      this.renderSubpage(this.Templates.view(this.templateData_));
    } else {
      this.renderSubpage(this.Templates.page());
    }

    this.renderHeader_();
  }

  onToogleConnect_() {
    proxy.isEnabled = !proxy.isEnabled;
  }

  onLocationSelected_(code) {
    if (this.isVisible) {
      geo.countryCode = code;
      this.currentPageID = this.defaultPageID;
    }
  }

  onNotificationChanged_() {
    if (this.isVisible) {
      if (this.isHomePageVisible) {
        this.currentPageID = this.defaultPageID;
      }
    } else {
      this.currentPageID = this.defaultPageID;
    }
  }

  onProxyChanged_() {
    if (this.currentPageID !== this.defaultPageID) {
      this.currentPageID = this.defaultPageID;
    } else if (this.isVisible) {
      this.render();
    }
  }

  onChanged_() {
    if (this.isVisible) {
      this.render();
    }
  }

  onCanShowChanged_() {
    if (this.isVisible && !this.isShowable) {
      this.canShow = false;
    } else if (this.isVisible && this.isShowable) {
      if (this.isHomePageVisible) {
        this.currentPageID = this.defaultPageID;
      } else if (this.currentPageID === IpPage.ID &&
          geo.userLocationUnknown) {
        this.currentPageID = this.defaultPageID;
      }
    } else {
      this.currentPageID = this.defaultPageID;
      this.canShow = this.isShowable;
    }
  }
};

AbstractHomeView.Templates = class extends AbstractPageView.Templates {
  static header(isHomePageVisible, pageID) {
    var header = super.header(!isHomePageVisible);

    if (pageID !== SideMenuPage.ID) {
      header.push([
        'div', {
          'class': 'button-power',
          'data-handler': AbstractHomeView.HANDLER_TOGGLE_CONNECT
        }
      ]);
    }

    return header;
  }

  static list(data, items) {
    items = [
      this.locationItem(data.server.location, data.isConnected),
      this.ipItem(data.server.ip, data.isConnected, data.hasUserIP)
    ].concat(items || []);

    return [
      'div', {
        'class': 'list'
      },
      items
    ];
  }

  static locationItem(location, isConnected) {
    if (!location) {
      return null;
    }

    return isConnected ?
        [
          'div', {
            'class': 'list-item is-link has-icon icon-location',
            'data-handler': AbstractPageView.HANDLER_NAVIGATE_TO_PAGE,
            'data-page': LocationPage.ID
          },
          location
        ] :
        [
          'div', {
            'class': 'list-item is-exposed'
          },
          i18n.getTemplate('homeLabelLocationExposed', { '$1': ['strong',
                                                                location] })
        ];
  }

  static ipItem(ip, isConnected, hasUserIP) {
    if (!ip) {
      return null;
    }

    return isConnected ?
        (hasUserIP ? [
          'div', {
            'class': 'list-item is-link has-icon icon-ip',
            'data-handler': AbstractPageView.HANDLER_NAVIGATE_TO_PAGE,
            'data-page': IpPage.ID
          },
          ip
        ] : ['div', {'class': 'list-item has-icon icon-ip'}, ip]) :
        [
          'div', {
            'class': 'list-item is-exposed'
          },
          i18n.getTemplate('homeLabelIPExposed', { '$1': ['strong', ip] })
        ];
  }

  static view(data) {
    return [
      [
        'header',
        this.page()
      ],
      this.list(data)
    ];
  }
};

AbstractHomeView.CLASS_NAME = 'home-view';
AbstractHomeView.HANDLER_TOGGLE_CONNECT = 'home-toggle-connect';
